'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getPlayers = getPlayers;
exports.getUserLinkedPlayers = getUserLinkedPlayers;
exports.getPlayer = getPlayer;
exports.getPlayerPrefix = getPlayerPrefix;
exports.writeOfflinePlayers = writeOfflinePlayers;

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _nodebb = require('../nodebb');

var _backend = require('../backend');

var _backend2 = _interopRequireDefault(_backend);

var _config = require('../config');

var _config2 = _interopRequireDefault(_config);

var _updater = require('../updater');

var _updater2 = _interopRequireDefault(_updater);

var _utils = require('../utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function updateUuids(players, callback) {
  var uuids = [];

  for (var i in players) {
    var player = players[i];

    if (!player.lastupdate || Date.now() - parseInt(player.lastupdate, 10) > _config2.default.getPlayerExpiry()) {
      uuids.push(player.id);
    }
  }

  _updater2.default.updateUuids(uuids);
  callback(null, players);
}

function addUserData(players, callback) {
  _async2.default.each(players, function (player, next) {
    // TODO: getMultipleUsersData?
    _nodebb.User.getUserData(player.uid, function (err, user) {
      if (user) player.user = user;
      next();
    });
  }, function () {
    callback(null, players);
  });
}

function getPlayers(data, callback) {
  _async2.default.waterfall([_async2.default.apply(_backend2.default.getUuids, data), _async2.default.apply(_backend2.default.getPlayersFromUuids), _async2.default.apply(updateUuids), _async2.default.apply(addUserData)], callback);
}

function getUserLinkedPlayers(uid, callback) {
  _async2.default.waterfall([_async2.default.apply(_backend2.default.getUuidsFromUid, uid), _async2.default.apply(_backend2.default.getPlayersFromUuids), function (players, next) {
    _backend2.default.getPrimaryUuid(uid, function (err, uuid) {
      for (var i in players) {
        if (players[i].id === uuid) players[i].isPrimary = true;
      }
      next(null, players);
    });
  }], callback);
}

function getPlayerFromUuid(uuid, callback) {
  _backend2.default.getPlayersFromUuids([uuid], function (err, players) {
    callback(err, players && players[0] ? players[0] : null);
  });
}

function getPlayerFromName(name, callback) {
  _async2.default.waterfall([_async2.default.apply(_backend2.default.getUuidFromName, name), _async2.default.apply(getPlayerFromUuid)], callback);
}

function getPlayerFromUid(name, callback) {
  _async2.default.waterfall([_async2.default.apply(_backend2.default.getUuidFromUid, name), _async2.default.apply(getPlayerFromUuid)], callback);
}

function getPlayer(data, callback) {
  if (!data || !(data.id || data.name || data.uid)) return callback(new Error('No data for player lookup.'));
  if (data.id) return getPlayerFromUuid(data.id, callback);
  if (data.name) return getPlayerFromName(data.name, callback);
  if (data.uid) return getPlayerFromUid(data.uid, callback);
}

// API.resetPrimaryUid = function (yuuid, next) {
// Backend.removePrimaryUid(yuuid, function () {
// Backend.getUidsFromUuid(yuuid, function (err, uids) {
// if (!err && uids && uids[0]) {
// db.setObjectField('yuuid:' + id, 'uid', uids[0], next)
// } else {
// next()
// }
// })
// })
// }

function getPlayerPrefix(data, callback) {
  if (!(data && (data.uuid || data.name))) return callback(new Error('Invalid params sent to API.getPlayerPrefix'));

  if (data.name) {
    _async2.default.waterfall([_async2.default.apply(_backend2.default.getUuidFromName, name), _async2.default.apply(_backend2.default.getPlayerPrefix)], callback);
  } else {
    _backend2.default.getPlayerPrefix(uuid, callback);
  }
}

function writeOfflinePlayers(players, callback) {
  for (var id in players) {
    console.log(id + ' ' + players[id].name);
  }
  callback();
}