'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateServerStatus = updateServerStatus;
exports.getServerStatus = getServerStatus;
exports.eventPlayerJoin = eventPlayerJoin;
exports.eventPlayerQuit = eventPlayerQuit;

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _config = require('../config');

var _config2 = _interopRequireDefault(_config);

var _backend = require('../backend');

var _backend2 = _interopRequireDefault(_backend);

var _controller = require('../controller');

var _controller2 = _interopRequireDefault(_controller);

var _utils = require('../utils');

var _utils2 = _interopRequireDefault(_utils);

var _nodebb = require('../nodebb');

var _users = require('./users');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// TODO
function updateServerStatus(status, next) {
  var updateTime = Math.round(Date.now() / 60000) * 60000,
      sid = status.sid,
      tps = status.tps;

  status.isServerOnline = '1';
  status.players = status.players || [];
  status.pluginList = status.pluginList || [];
  status.hasPlugins = status.pluginList.length;
  status.modList = status.modList || [];
  status.hasMods = status.modList.length;
  status.updateTime = updateTime;

  // Sort Plugins
  status.pluginList = status.pluginList.sort(function (a, b) {
    return a.name.toLowerCase() > b.name.toLowerCase() ? 1 : -1;
  });

  // Trim UUIDs to Mojang format.
  status.players.forEach(function (player) {
    if (!player.id) return;
    player.id = _utils2.default.trimUUID(player.id);
  });

  // Store the player statistics in the database.
  _async2.default.each(status.players, function (player, next) {

    // Skip if no uuid.
    if (!player.id) next();

    // TODO: BungeeCord Support
    // BungeeCord proxies will need their player UUIDs verified, since they run in offline mode.
    // A proper proxy would be running a Spigot derivative with BungeeCord pass-through set, but
    // we can't count on that.

    // Skip if invalid uuid.
    // function verifyUUID(id, next) {
    // Utils.getPlayerNameUsingUUID(id, function (err, name) {
    // if (err) return next(err)
    // })
    // }

    // Utils.getPlayerNameUsingUUID(player.id, function (err, valid) {
    // if (err)
    // })

    // DEPRECIATED: Future versions will track playtime using the Minecraft Plugin or OnTime.
    _async2.default.parallel({
      playtime: function playtime(next) {
        _nodebb.db.getObjectField('yuuid:' + player.id, 'lastonline', function (err, data) {
          if (parseInt(data) !== updateTime) {
            _nodebb.db.setObjectField('yuuid:' + player.id, 'lastonline', updateTime);
            _nodebb.db.incrObjectField('yuuid:' + player.id, 'playtime', next);
          } else {
            _nodebb.db.getObjectField('yuuid:' + player.id, 'playtime', next);
          }
        });
      },

      name: _async2.default.apply(_nodebb.db.setObjectField, 'yuuid:' + player.id, 'name', player.name)
    }, function (err, results) {
      if (err) {
        console.log('[Minecraft Integration] Error setting player object ' + player.id + ': ' + err);
      } else {
        _nodebb.db.sortedSetAdd('yuuid:playtime', results.playtime || '0', player.id, function (err) {});
      }
    });
  });

  // wth is this?
  _async2.default.waterfall([_async2.default.apply(_backend2.default.updateServerStatus, status), function (next) {
    getServerStatus({ sid: status.sid }, function (err, status) {
      _controller2.default.sendStatusToUsers(status);
    });
    next();
  }], next);
}

// TEMP


function getServerStatus(data, callback) {
  if (!(data && typeof data.sid !== 'undefined')) return callback(new Error('Invalid data.'));

  var sid = data.sid;

  _async2.default.parallel({
    status: _async2.default.apply(_backend2.default.getServerStatus, sid),
    config: _async2.default.apply(_backend2.default.getServerConfig, { sid: sid })
  }, function (err, results) {
    if (err) return callback(err);

    var status = results.status;
    var config = results.config;

    if (!config) return callback(new Error('getServerStatus() No config exists for SID ' + sid));
    if (!status) return callback(new Error('getServerStatus() No status exists for SID ' + sid + ' named ' + config.name));

    // Parsed as arrays.
    try {
      if (status.players && typeof status.players === 'string' && status.players !== 'undefined') status.players = JSON.parse(status.players);
      if (status.modList && typeof status.modList === 'string' && status.modList !== 'undefined') status.modList = JSON.parse(status.modList);
      if (status.pluginList && typeof status.pluginList === 'string' && status.pluginList !== 'undefined') status.pluginList = JSON.parse(status.pluginList);
    } catch (e) {
      console.log('Bad Status', status);
      return callback(e);
    }

    // Parsed as integers.
    status.maxPlayers = parseInt(status.maxPlayers, 10);
    status.onlinePlayers = parseInt(status.onlinePlayers, 10);

    // Parsed as booleans.
    status.isServerOnline = !!parseInt(status.isServerOnline, 10);
    status.hasMods = !!parseInt(status.hasMods, 10);
    status.hasPlugins = !!parseInt(status.hasPlugins, 10);

    // Strings
    status.sid = sid;
    status.address = config.address;
    status.version = _utils2.default.parseVersion(status.version || 'unknown');

    // Hide plugins.
    if (parseInt(config.hidePlugins, 10)) status.pluginList = [];

    callback(null, status);
  });
}

function eventPlayerJoin(data, callback) {

  // Assert parameters.
  if (!(data && data.id && data.name)) return callback();

  var name = data.name;
  var nick = data.nick;
  var id = data.id;
  var prefix = data.prefix;
  var suffix = data.suffix;
  var groups = data.groups;
  var playtime = data.playtime;

  var usePrimaryPrefixOnly = _config2.default.settings.get('usePrimaryPrefixOnly');
  var newPrefix = '';
  var primaryPrefix = '';

  // Parse prefix data.
  // TODO: Configure spacing.
  if (prefix) {
    newPrefix = _utils2.default.parseFormatCodes(prefix);
    primaryPrefix = newPrefix;
  }
  if (Array.isArray(groups)) {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = groups[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        // TODO: Get prefix from group hash.

        var group = _step.value;
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  }

  _nodebb.db.setObjectField('yuuid:' + id, 'prefix', usePrimaryPrefixOnly ? primaryPrefix : newPrefix);

  _async2.default.parallel({
    player: function player(next) {

      // TODO:
      // I can get rid off all this nonsense by storing players in a set 'mi:server:sid:players'
      // Could use a sortedSet with lexicon stored names, or just a plain set with uuids.
      _nodebb.db.getObjectField('mi:server:' + data.sid, 'players', function (err, players) {
        if (err) return console.log(err);

        var found = false;

        try {
          players = JSON.parse(players);
        } catch (e) {
          return console.log('Bad Players Object', e);
        }

        for (var i in players) {
          if (players[i] === null) continue;
          if (players[i].id === id) found = true;
        }
        if (!found) {
          players.push({ id: id, name: name });

          try {
            players = JSON.stringify(players);
          } catch (e) {
            return console.log(e);
          }

          // Updates widgets with new player.
          _controller2.default.sendPlayerJoinToUsers({ sid: data.sid, player: { id: id, name: name } });

          _nodebb.db.setObjectField('mi:server:' + data.sid, 'players', players, function (err) {
            if (err) console.log(err);
            next();
          });
        } else {
          next();
        }
      });
    },
    user: function user(next) {
      // TEMP
      (0, _users.getUser)({ id: id }, function (err, user) {
        next(null, user || null);
      });
    }
  }, callback);
}

function eventPlayerQuit(data, next) {

  // Assert parameters.
  if (!(data && data.id && data.name)) return next();

  var name = data.name,
      id = data.id;

  _nodebb.db.getObjectField('mi:server:' + data.sid, 'players', function (err, players) {
    if (err) {
      console.log(err);
      return next();
    }

    try {
      players = JSON.parse(players);
    } catch (e) {
      console.log(e);
      return next();
    }

    // TODO: Make this part not suck.
    for (var i in players) {
      if (players[i].id === id) {
        players.splice(i, 1);
      }
    }
    try {
      players = JSON.stringify(players);
      _nodebb.db.setObjectField('mi:server:' + data.sid, 'players', players, function (err) {
        if (err) return console.log(err);
      });
      _controller2.default.sendPlayerQuitToUsers({ sid: data.sid, player: { id: id, name: name } });
    } catch (e) {
      console.log(e);
    }

    return next();
  });
}