'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resetPlayerKey = resetPlayerKey;
exports.getPlayerKey = getPlayerKey;
exports.getPlayerKeyUID = getPlayerKeyUID;
exports.getUsers = getUsers;
exports.getUser = getUser;
exports.deleteUser = deleteUser;
exports.refreshUser = refreshUser;
exports.resetUsers = resetUsers;
exports.getUserPrefix = getUserPrefix;

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _nodebb = require('../nodebb');

var _backend = require('../backend');

var _backend2 = _interopRequireDefault(_backend);

var _config = require('../config');

var _config2 = _interopRequireDefault(_config);

var _updater = require('../updater');

var _updater2 = _interopRequireDefault(_updater);

var _utils = require('../utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function resetPlayerKey(data, cb) {
  cb = cb || function () {};

  if (!(data && data.uid)) return cb(new Error('Bad data sent to Backend.resetPlayerKey.'));

  var uid = parseInt(data.uid, 10);

  // Reset all keys with uid.
  _nodebb.db.sortedSetsRemoveRangeByScore(['playerkey:uid'], uid, uid, function () {
    getPlayerKey(data, cb);
  });
}

// Gets and/or creates the player key.
function getPlayerKey(data, cb) {
  cb = cb || function () {};

  if (!(data && data.uid)) return cb(new Error('Bad data sent to API.getPlayerKey.'));

  var uid = data.uid;

  _nodebb.db.getSortedSetRangeByScore('playerkey:uid', 0, 1, uid, uid, function (err, key) {
    if (err || !key || !key.length) {
      key = _utils2.default.getKey();
      _nodebb.db.sortedSetAdd('playerkey:uid', uid, key);
    } else {
      key = key[0];
    }
    return cb(err, { key: key });
  });
}

function getPlayerKeyUID(data, cb) {
  if (!(data && data.key)) return cb(new Error('Bad data sent to Backend.getPlayerKeyUID.'));

  _nodebb.db.sortedSetScore('playerkey:uid', data.key, function (err, uid) {
    if (err || !uid) err = err || new Error('FAILKEY');

    return cb(err, uid);
  });
}

// Get all linked users.
function getUsers(options, next) {
  var fields = ['uid', 'username', 'yuuid', 'picture'];

  if (options.fields && Array.isArray(options.fields)) {
    for (var i = 0; i < options.fields.length; i++) {
      if (typeof options.fields[i] === 'string') fields.push(options.fields[i]);
    }
  }

  _backend2.default.getLinkedUids(function (err, uids) {
    _nodebb.User.getUsersFields(uids, fields, function (err, usersData) {
      _async2.default.map(usersData, function (userData, next) {
        _backend2.default.getUuidsFromUid(userData.uid, function (err, uuids) {
          if (err || !uuids) return next(null, userData); // TODO: Remove from sortedset if links are missing from db.
          _backend2.default.getPlayersFromUuids(uuids, function (err, players) {
            userData.players = players.map(function (player) {
              if (player.id === userData.yuuid) player.isPrimary = true;
              return player;
            });
            next(null, userData);
          });
        });
      }, next);
    });
  });
}

// Get the primary linked user from uid, yuuid, or name.
function getUser(options, next) {

  // Assert parameters
  if (!(typeof next === 'function' && options && (options.uid || options.id || options.name))) return next('Backend.getUser() Invalid params.');

  // Convert name to uuid if needed.
  if (options.id) {
    getUserFromUuid(options.id, options.extraFields, next);
  } else if (options.name) {
    _backend2.default.getPlayerFromName(options.name, function (err, profile) {
      if (err || !profile) return next(err, profile);
      getUserFromUuid(profile.id, options.extraFields, next);
    });
  }
}

function getUserFromUuid(id, extraFields, next) {

  // Get primary user.
  _nodebb.db.getObjectField('yuuid:' + id, 'uid', function (err, uid) {
    if (err || !uid) return next(err);
    _nodebb.User.getUserData(uid, function (err, userData) {
      if (err || !userData) return next(err);
      // Backend.getPlayerFromUuid(id, function (err, player) {
      // if (err || !player) return next(err)

      // TEMP
      // player.user = undefined
      // userData.player = player
      // next(null, userData)
      // })
      next(null, userData);
    });
  });
}

function deleteUser(data, next) {
  if (!data || !data.uid) return next(new Error('No uid'));

  var uid = parseInt(data.uid, 10);

  // Backend.sortedSetRemove()
  // Backend.getUuidsFromUid( uid, function (yuuids, next) {
  // async.each(yuuids, function (yuuid, next) {
  // Backend.getUidsFromUuid(yuuid, function (err, uids) {
  // Backend.getPrimaryUid(yuuid, function (err, primaryuid) {
  // if (!err && primaryuid && parseInt(primaryuid, 10) === uid) {
  // db.getSortedSetRange('yuuid:' + yuuid  + ':uids', 0, -1, function (err, uids) {
  // })
  // }
  // })
  // })
  // })
  // })

  // async.parallel([
  // Get all uuids, get primary uids, reset primary uid to something else if equal, remove from set.

  // // async.apply(db.deleteObjectField, 'yuuid:' + id,  'uid'),
  // async.apply(db.sortedSetRemove,   'yuuids:linked', id),
  // function (next) {
  // db.getSortedSetRange('yuuid:' + id  + ':uids', 0, -1, function (err, uids) {
  // async.each(uids, function (uid, next) {
  // async.parallel([
  // async.apply(db.sortedSetRemove, 'yuuid:' + id  + ':uids',   uid),
  // async.apply(db.sortedSetRemove, 'uid:'   + uid + ':yuuids', id)
  // ], next)
  // }, next)
  // })
  // }
  // ], next)
}

function refreshUser(data, next) {}

function resetUsers(data, next) {}

function getUserPrefix(data, next) {
  if (!(data && data.uid)) return next(new Error('Invalid params sent to API.getUserPrefix'));
  _backend2.default.getPrimaryUuid(data.uid, function (err, uuid) {
    if (err || !uuid) return next(err);
    _backend2.default.getPlayerPrefix(uuid, next);
  });
}