'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _nodebb = require('./nodebb');

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Backend = module.exports = {};

// ///////////////////
// Player Objects  //
// yuuid:{uuid}    // Hash for player object.
// yuuid:sorted    // SortedSet value:yuuid score:lastpinged
// yuuid:playtime  // SortedSet value:yuuid score:playtime
// ///////////////////

// TODO: Need to remove console.log for winston.

// Get all stored player uuids.
Backend.getUuids = function (options, next) {
  options = options || {};

  var set = options.set || 'yuuid:playtime',
      min = options.min || 0,
      max = options.max || -1,
      profiles = [];

  _nodebb.db.getSortedSetRevRange(set, min, max, next);
};

// Get the player hash of uuids.
Backend.getPlayersFromUuids = function (yuuids, callback) {
  var keys = yuuids.map(function (yuuid) {
    return 'yuuid:' + yuuid;
  });

  _nodebb.db.getObjects(keys, function (err, players) {

    // Add the yuuid to the player object for display.
    for (var i in players) {
      players[i] = players[i] || {};
      players[i].id = yuuids[i];
    }

    callback(err, players);
  });
};
Backend.getPlayerFromUuid = function (uuid, callback) {
  Backend.getPlayersFromUuids([uuid], function (err, players) {
    if (err || !(players && players[0])) return callback(err);
    callback(null, players[0]);
  });
};

Backend.getUuidFromName = function (name, next) {
  // Look in the cache first.
  _nodebb.db.get('mi:name:' + name, function (err, uuid) {

    // Return if db error.
    if (err) return next(err);

    // Return if found in cache.
    if (uuid) return next(null, uuid);

    // If not in cache, query Mojang.
    _utils2.default.getUUID(name, function (err, uuid) {

      // Return if db error.
      if (err) return next(err);

      // Store results in the cache.
      _nodebb.db.set('mi:name:' + name, uuid);
      _nodebb.db.expire('mi:name:' + name, _config2.default.getPlayerExpiry());

      // Return the UUID.
      return next(null, uuid);
    });
  });
};

// Get the primary player linked to a user.
Backend.getPrimaryUuid = function (uid, next) {
  _nodebb.db.getObjectField('user:' + uid, 'yuuid', next);
};

// Get the primary user linked to a player.
Backend.getPrimaryUid = function (yuuid, next) {
  _nodebb.db.getObjectField('yuuid:' + yuuid, 'uid', next);
};

// Get the all players linked to a user.
Backend.getUuidsFromUid = function (uid, next) {
  _nodebb.db.getSortedSetRevRange('uid:' + uid + ':yuuids', 0, -1, next);
};

// Get the all users linked to a player.
Backend.getUidsFromUuid = function (yuuid, next) {
  _nodebb.db.getSortedSetRevRange('yuuid:' + yuuid + ':uids', 0, -1, next);
};

// Remove primary user linked to a player.
Backend.removePrimaryUid = function (yuuid, next) {
  _nodebb.db.deleteObjectField('yuuid:' + yuuid, 'uid', next);
};

// Remove primary player linked to a user.
Backend.removePrimaryUuid = function (uid, next) {
  _nodebb.db.deleteObjectField('user:' + uid, 'yuuid', next);
};

// Add link to sortedsets and primaries.
Backend.linkUuidtoUid = function (yuuid, uid, next) {
  _async2.default.parallel([_async2.default.apply(_nodebb.db.sortedSetAdd, 'mi:uid:linked', Date.now(), uid), _async2.default.apply(_nodebb.db.sortedSetAdd, 'yuuid:' + yuuid + ':uids', Date.now(), uid), _async2.default.apply(_nodebb.db.sortedSetAdd, 'yuuid:linked', Date.now(), yuuid), _async2.default.apply(_nodebb.db.sortedSetAdd, 'uid:' + uid + ':yuuids', Date.now(), yuuid), _async2.default.apply(_nodebb.db.setObjectField, 'yuuid:' + yuuid, 'uid', uid), _async2.default.apply(_nodebb.db.setObjectField, 'user:' + uid, 'yuuid', yuuid)], next);
};

Backend.resetPrimaryUid = function (yuuid, next) {
  Backend.removePrimaryUid(yuuid, function () {
    Backend.getUidsFromUuid(yuuid, function (err, uids) {
      if (!err && uids && uids[0]) {
        _nodebb.db.setObjectField('yuuid:' + id, 'uid', uids[0], next);
      } else {
        next();
      }
    });
  });
};

Backend.resetPrimaryUuid = function (uid, next) {
  Backend.removePrimaryUuid(uid, function () {
    Backend.getUuidsFromUid(uid, function (err, uuids) {
      if (!err && uuids && uids[0]) {
        _nodebb.db.setObjectField('user:' + id, 'uid', uids[0], next);
      } else {
        next();
      }
    });
  });
};

Backend.getLinkedUids = function (next) {
  _nodebb.db.getSortedSetRevRange('mi:uid:linked', 0, -1, next);
};
Backend.getLinkedUuids = function (next) {
  _nodebb.db.getSortedSetRevRange('yuuid:linked', 0, -1, next);
};

Backend.deletePlayer = function (data, next) {
  if (!(data && data.id)) return next(new Error('Bad data sent to Backend.deletePlayer()'));

  var id = data.id;

  _async2.default.parallel([_async2.default.apply(_nodebb.db.delete, 'yuuid:' + id),
  // async.apply(db.deleteObjectField, 'yuuid:' + id,  'uid'),
  _async2.default.apply(_nodebb.db.sortedSetRemove, 'yuuids:linked', id), function (next) {
    _nodebb.db.getSortedSetRange('yuuid:' + id + ':uids', 0, -1, function (err, uids) {
      _async2.default.each(uids, function (uid, next) {
        _async2.default.parallel([_async2.default.apply(_nodebb.db.sortedSetRemove, 'yuuid:' + id + ':uids', uid), _async2.default.apply(_nodebb.db.sortedSetRemove, 'uid:' + uid + ':yuuids', id)], next);
      }, next);
    });
  }], next);
};

Backend.getPlayerPrefix = function (uuid, next) {
  _nodebb.db.getObjectField('yuuid:' + uuid, 'prefix', next);
};

// /////////////////////////////////
// Server Status                 //
// /////////////////////////////////
// mi:server:{sid}               // Object
// mi:server:{sid}:pings         // List of JSON strings
// mi:server:{sid}:players       // SortedSet of UUIDs by playtime.
// mi:server:{sid}:player:{uuid} // Object of player statistics.
// mi:server:{sid}:config        // Object of server configuration.
// /////////////////////////////////

Backend.getServer = Backend.getServerStatus = function (sid, callback) {
  _nodebb.db.getObject('mi:server:' + sid, callback);
};

function sort(a, b) {
  return a - b;
}

// Exposed to admin api only.
Backend.getServerConfig = function (data, next) {
  if (!(data && data.sid)) return next(new Error('Backend.getServerConfig() No SID.', data));

  var sid = data.sid;

  _nodebb.db.getObject('mi:server:' + sid + ':config', function (err, config) {
    if (err) return next(err);
    if (!config) return next(new Error('Backend.getServerConfig() invalid SID: ' + data.sid));

    config.sid = sid;

    next(null, config);
  });
};

Backend.setServerConfig = function (data, next) {
  if (!(data && data.sid)) return next(new Error('Backend.setServerConfig() No SID.'));

  data.name = data.name || 'Unnamed Server';
  data.address = data.address || 'example.com';

  // Score is used for sorting.
  _nodebb.db.sortedSetAdd('mi:servers', Date.now(), data.sid);
  _nodebb.db.setObject('mi:server:' + data.sid + ':config', data.config, next);
};

Backend.getSidUsingAPIKey = function (key, next) {
  var payload = null;

  Backend.getServersConfig({}, function (err, configs) {
    if (err) return next(err);

    configs.forEach(function (config) {
      if (config.APIKey === key) payload = config.sid;
    });

    return next(payload ? null : new Error('Invalid API Key'), payload);
  });
};

Backend.getServerIcon = function (data, next) {

  if (!data || !data.sid) return next('Backend.getServerIcon() No SID.');

  var sid = data.sid;

  _nodebb.db.getObjectField('mi:server:' + sid, 'icon', function (err, icon) {
    if (err || !icon) return next(err);

    var base = icon.replace('data:image/png;base64,', '');

    next(err, {
      base: base,
      modified: new Date().toUTCString()
    });
  });
};

Backend.getServerPlugins = function (data, next) {

  var sid = data.sid;

  _async2.default.parallel({
    pluginList: _async2.default.apply(_nodebb.db.getObjectField, 'mi:server:' + sid, 'pluginList'),
    config: _async2.default.apply(Backend.getServerConfig, { sid: sid })
  }, function (err, results) {

    if (err) return next(err);

    if (!results.pluginList || parseInt(results.config.hidePlugins, 10)) {
      next(null, []);
    } else {
      try {
        results.pluginList = JSON.parse(results.pluginList);
      } catch (err) {
        console.log('JSON ERROR: ' + err);
        return next(err, []);
      }

      next(null, results.pluginList);
    }
  });
};

// TODO: Make this retrieve a time range instead of a fixed amount.
Backend.getRecentPings = function (data, next) {

  var pings = [];
  var sid = data.sid;
  var amount = data.last || 30;
  var stamps = [];
  var now = Date.now();

  if (sid < 0) return next(new Error('Invalid sid sent to getRecentPings: ' + sid), pings);

  // Get stamps of the last 'amount' minutes.
  now = now - now % 60000;

  _nodebb.db.getObject('mi:server:' + sid + ':ping:' + now, function (err, ping) {
    if (!ping) now -= 60000;

    for (var i = 0; i < amount + 1; i++) {
      stamps.push(now - i * 60000);
    }stamps = stamps.reverse();

    // Read and parse the stored pings for each stamp.
    _async2.default.map(stamps, function (stamp, next) {

      _nodebb.db.getObject('mi:server:' + sid + ':ping:' + stamp, function (err, ping) {
        if (err) return next(err);

        if (!ping && stamps[stamps.length - 1] === stamp) stamp = { players: -1 };

        // Make missing pings blank instead of erroring.
        ping = ping || defaultPing;
        for (var p in defaultPing) {
          ping[p] = ping[p] || defaultPing[p];
        } // TODO: Read players as a separate hash.
        if (typeof ping.players === 'string') {
          try {
            ping.players = JSON.parse(ping.players);
          } catch (e) {
            ping.players = [];
          }
        }

        // Remove invalid TPS values.
        if (parseInt(ping.tps, 10) > 100) ping.tps = '0';

        // Store stamp for charting, and human time for display.
        ping.timestamp = stamp;
        ping.humanTime = _utils2.default.getHumanTime(stamp);

        next(null, ping);
      });
    }, next);
  });
};

var defaultPing = {
  players: [],
  tps: 0
};

Backend.updateServerStatus = function (status, next) {

  if (typeof status.players !== 'string') status.players = JSON.stringify(status.players);

  if (status.pluginList && typeof status.pluginList !== 'string') status.pluginList = JSON.stringify(status.pluginList);
  if (status.modList && typeof status.modList !== 'string') status.modList = JSON.stringify(status.modList);

  _async2.default.waterfall([_async2.default.apply(_nodebb.db.delete, 'mi:server:' + status.sid), _async2.default.apply(_nodebb.db.setObject, 'mi:server:' + status.sid, status), _async2.default.apply(_nodebb.db.expire, 'mi:server:' + status.sid, _config2.default.getPingExpiry()), _async2.default.apply(_nodebb.db.setObjectField, 'mi:server:' + status.sid + ':ping:' + status.updateTime, 'players', status.players), _async2.default.apply(_nodebb.db.setObjectField, 'mi:server:' + status.sid + ':ping:' + status.updateTime, 'tps', status.tps), _async2.default.apply(_nodebb.db.expire, 'mi:server:' + status.sid + ':ping:' + status.updateTime, _config2.default.getPingExpiry()), _async2.default.apply(updatePingList, 'mi:server:' + status.sid + ':pings', status.updateTime)], next);
};

function updatePingList(key, value, cb) {
  _nodebb.db.getListRange(key, 0, 0, function (err, values) {
    if (err) return cb(err);

    // Is the most recent ping stamp empty or outdated?
    if (!values || !values[0] || !values[0] === value) {
      _nodebb.db.listPrepend(key, value);
    }

    cb();
  });
}

Backend.deleteServer = function (data, next) {
  if (!data || !(parseInt(data.sid, 10) > -1)) return next(new Error('Invalid data sent to Backend.deleteServer()'));

  var sid = parseInt(data.sid, 10);

  _winston2.default.warn('DELETING MINECRAFT SERVER SID:' + sid);

  _async2.default.waterfall([_async2.default.apply(_nodebb.db.getListRange, 'mi:server:' + sid + ':pings', 0, -1), function (stamps, next) {
    _async2.default.each(stamps, function (stamp, next) {
      _nodebb.db.delete('mi:server:' + sid + ':ping:' + stamp, next);
    }, next);
  }, _async2.default.apply(_nodebb.db.delete, 'mi:server:' + sid + ':pings'), _async2.default.apply(_nodebb.db.delete, 'mi:server:' + sid), _async2.default.apply(_nodebb.db.delete, 'mi:server:' + sid + ':config'), _async2.default.apply(_nodebb.db.sortedSetRemove, 'mi:servers', sid)
  // mi:server:{sid}:players       // SortedSet of UUIDs by playtime.
  // mi:server:{sid}:player:{uuid} // Object of player statistics.
  ], next);
};

// //////////////////////////////
// Servers                    //
// //////////////////////////////
// mi:servers                 //
// mi:servers:apikeys         //
// //////////////////////////////

Backend.getServers = Backend.getServersStatus = function (data, next) {
  Backend.getServersSids(data, function (err, sids) {
    _async2.default.map(sids, function (sid, next) {
      Backend.getServer({ sid: sid }, function (err, server) {
        if (err) _winston2.default.error(err);
        next(null, server);
      });
    }, function (err, servers) {
      next(null, servers.filter(function (server) {
        return server;
      }));
    });
  });
};

// Exposed to admin api only.
Backend.getServersConfig = function (data, callback) {
  var payload = [];

  Backend.getServersSids(data, function (err, sids) {
    _async2.default.each(sids, function (sid, next) {
      Backend.getServerConfig({ sid: sid }, function (err, config) {
        if (err) return next();
        if (!config) return next();
        payload.push(config);
        next();
      });
    }, function () {
      callback(null, payload);
    });
  });
};

Backend.getServersSids = Backend.getServerSids = function (data, next) {
  if (!data || (typeof data === 'undefined' ? 'undefined' : _typeof(data)) !== 'object') return next(new Error('No data sent to Backend.getServersSids()'));

  if (data.start) data.start = parseInt(data.start, 10);
  if (data.end) data.start = parseInt(data.end, 10);

  var start = data.start >= 0 ? data.start : 0;
  var end = data.end >= -1 ? data.end : -1;

  _nodebb.db.getSortedSetRange('mi:servers', 0, -1, function (err, sids) {

    if (err || !sids) return next(err, []);

    if (data.sort) {
      if (data.sort === 'sid') sids = sids.sort(sort);
    } else {
      sids = sids.sort(sort);
    }

    next(err, sids);
  });
};

// ///////////////////////////////////
// Avatars                         //
// ///////////////////////////////////
// mi:avatars                      // SortedSet, value: player name of stored avatar, score: last update time.
// mi:avatar:{playername}          // String, base64 encoded png.
// mi:avatar:{playername}:modified // String, ISO Date of last time the image was modified.
// ///////////////////////////////////

// ACP list
Backend.getAvatars = function (data, next) {
  Backend.getAvatarList({}, function (err, avatarList) {
    var avatars = [];

    // TODO: This needs to be a multi-key operation.
    _async2.default.each(avatarList, function (name, next) {
      getAvatar(name, function (err, base64) {
        if (err || !base64) return next();

        Backend.getUuidFromName(name, function (err, uuid) {
          if (err || !uuid) return next();

          avatars.push({ name: name, base64: base64, id: uuid });
          next();
        });
      });
    }, function (err) {
      next(err, avatars);
    });
  });
};

// Get the avatar base64 from the database.
function getAvatar(name, callback) {

  // Database keys used.
  var keyBase = 'mi:avatar:' + name,
      keyModified = 'mi:avatar:' + name + ':modified',
      keySorted = 'mi:avatars';

  // Store a fetched avatar binary as base64 and update fetch time.
  function storeAvatar(avatar, next) {

    // Convert buffer to a base64.
    avatar = avatar.toString('base64');

    // Set base64.
    _nodebb.db.set(keyBase, avatar, function (err) {
      if (err) return next(err);

      return next(null, avatar);
    });

    // Update the avatar fetch time.
    _nodebb.db.sortedSetAdd(keySorted, Date.now(), name);
  }

  // Get fetch time.
  // If old or null, Fetch avatar and update fetch time.
  // If different, Set modified.
  _async2.default.parallel({
    fetchTime: _async2.default.apply(_nodebb.db.sortedSetScore, 'mi:avatars', name),
    base: _async2.default.apply(_nodebb.db.get, 'mi:avatar:' + name),
    modifiedTime: _async2.default.apply(_nodebb.db.get, 'mi:avatar:' + name + ':modified')
  }, function (err, results) {
    if (err) return callback(err);

    var fetchTime = results.fetchTime;
    var base = results.base;
    var modifiedTime = results.modifiedTime;
    var buffer = void 0;

    _async2.default.waterfall([function (next) {
      if (!fetchTime || !base || !modifiedTime || Date.now() - fetchTime > 1000 * 60 * 10) {
        fetchAvatar(name, next);
      } else {
        next(null, false);
      }
    }, function (_buffer, next) {
      if (_buffer) {
        buffer = _buffer;
        storeAvatar(buffer, next);
      } else {
        next(null, base);
      }
    }, function (_base, next) {
      if (_base !== base || !modifiedTime) {
        modifiedTime = new Date().toUTCString();
        _nodebb.db.set(keyModified, modifiedTime, next);
        base = _base;
      } else {
        next();
      }
    }], function (err) {
      if (err) return callback(err);

      callback(err, {
        buffer: buffer,
        base: base,
        modified: modifiedTime
      });
    });
  });
}

Backend.getAvatar = function (data, callback) {

  // Asserts
  if (!(data && data.name && typeof data.name === 'string')) return callback(new Error('Invalid Data passed to getAvatar: ' + data));

  getAvatar(data.name, callback);
};

// Get list of avatar names in the database
Backend.getAvatarList = function (data, callback) {
  _nodebb.db.getSortedSetRange('mi:avatars', 0, -1, function (err, list) {
    callback(err, list ? list.sort() : []);
  });
};

Backend.clearOldAvatars = function (options, next) {
  _nodebb.db.sortedSetsRemoveRangeByScore(['mi:avatars'], 0, Date.now() - _config2.default.getAvatarExpiry() * 1000, function (err) {
    if (err) console.log('Backend.clearOldAvatars error:', err);
    if (typeof next === 'function') next();
  });
};

Backend.refreshAvatar = function (data, next) {

  var name = data.name;

  Backend.deleteAvatar(data, function (err) {
    getAvatar(name, function (err, data) {
      next(err, { base64: data.base });
    });
  });
};

Backend.deleteAvatar = function (data, next) {
  if (!data || !data.name) return next(new Error('Backend.deleteAvatar() no name passed.'));

  var name = data.name;

  _nodebb.db.sortedSetRemove('mi:avatars', name);

  next();
};

Backend.resetAvatars = function (data, callback) {
  Backend.getAvatarList({}, function (err, avatarList) {
    _async2.default.each(avatarList, function (player, next) {
      var key = 'mi:avatar:' + player;

      _nodebb.db.delete(key);
      _nodebb.db.sortedSetRemove('mi:avatars', player);

      next();
    }, function (err) {
      callback(err);
    });
  });
};

Backend.setAvatar = function (data) {};

// Gets the avatar from the configured cdn.
function fetchAvatar(name, next) {
  _async2.default.parallel({
    url: _async2.default.apply(_config2.default.getAvatarUrl, { name: name, size: 64 }), // The full url for the avatar.
    id: _async2.default.apply(Backend.getUuidFromName, name) // We need this for cdns that use uuids.
  }, function (err, payload) {

    if (err) return next(err);

    var url = payload.url.replace('{uuid}', payload.id);

    console.log('Fetching avatar from CDN: ' + url);

    _async2.default.waterfall([_async2.default.apply(_request2.default, { url: url, encoding: null }), _async2.default.apply(transform)], function (err, avatar) {
      if (err) {
        console.log('Could not retrieve skin using the cdn: ' + _config2.default.settings.get('avatarCDN'));
        if (_config2.default.settings.get('avatarCDN') === 'mojang') return next(null, _config2.default.steveBuffer);

        // Try Mojang
        _async2.default.waterfall([_async2.default.apply(_request2.default, { url: 'http://skins.minecraft.net/MinecraftSkins/' + name + '.png', encoding: null }), function (response, body, next) {
          console.log('Defaulting to Mojang skin.');
          _config2.default.cdns['mojang'].styles.flat.transform(body, next);
        }], function (err, avatar) {
          if (err) {
            console.log("Couldn't connect to Mojang skin server.");

            return next(null, _config2.default.steveBuffer);
          } else {
            next(null, new Buffer(avatar));
          }
        });
      } else {
        next(null, new Buffer(avatar));
      }
    });
  });
}

function transform(response, body, next) {
  var cdn = _config2.default.settings.get('avatarCDN');
  if (_config2.default.cdns[cdn].styles && _config2.default.cdns[cdn].styles.flat && _config2.default.cdns[cdn].styles.flat.transform) {
    _config2.default.cdns[cdn].styles.flat.transform(body, next);
  } else {
    next(null, body);
  }
}

// /////////////////////
// Global Stats      //
// /////////////////////

Backend.getPlaytimes = function (options, next) {
  _nodebb.db.getSortedSetRangeWithScores('yuuid:playtime', 0, -1, function (err, data) {
    next(err, data);
  });
};

Backend.getTopPlayersByPlaytimes = function (data, callback) {
  data.show = data.show || 5;

  _nodebb.db.getSortedSetRevRangeByScoreWithScores('yuuid:playtime', 0, data.show, '+inf', 0, function (err, data) {
    _async2.default.map(data, function (value, next) {
      Backend.getPlayerFromUuid(value.value, function (err, profile) {
        if (err) return next(err);
        next(null, { id: value.value, name: profile.name, playtime: value.score, playtimeHuman: _utils2.default.parseMinutesDuration(value.score) });
      });
    }, callback);
  });
};

// Ranks

Backend.getRanks = function (sid, callback) {
  _nodebb.db.getSortedSetRange('mi:server:' + sid + ':ranks', 0, -1, callback);
};

Backend.getRankMembers = function (sid, rank, page, count, callback) {
  var key = 'mi:server:' + sid + ':rank:' + rank + ':members';

  if (typeof page === 'function') {
    callback = page;
    page = 0;
    count = 10000000;
  }

  _nodebb.db.getSortedSetRangeByLex(key, '-', '+', page * count, count, callback);
};

Backend.getRanksWithMembers = function (sid, page, count, callback) {
  var key = 'mi:server:' + sid + ':ranks';

  if (typeof page === 'function') {
    callback = page;
    page = 0;
    count = 10000000;
  }

  var data = [];

  _async2.default.waterfall([_async2.default.apply(Backend.getRanks, sid), function (ranks, next) {
    _async2.default.each(ranks, function (rank, next) {
      Backend.getRankMembers(sid, rank, page, count, function (err, members) {
        if (err) return next(err);
        data.push({
          name: rank,
          members: members.map(function (member) {
            return { id: member };
          })
        });
        next();
      });
    }, next);
  }], function (err) {
    callback(err, data);
  });
};

Backend.setRanks = function (sid, ranks, callback) {
  var key = 'mi:server:' + sid + ':ranks';

  _async2.default.waterfall([_async2.default.apply(_nodebb.db.delete, key), _async2.default.apply(_async2.default.each, ranks, function (rank, next) {
    _nodebb.db.sortedSetAdd(key, rank.rank || 0, rank.name, next);
  })], function (err) {
    callback(err);
  });
};

Backend.setRanksWithMembers = function (sid, ranks, callback) {
  _async2.default.waterfall([_async2.default.apply(Backend.setRanks, sid, ranks), _async2.default.apply(_async2.default.each, ranks, function (rank, next) {
    var key = 'mi:server:' + sid + ':rank:' + rank.name + ':members';

    _nodebb.db.delete(key, function () {
      _async2.default.each(rank.members, function (member, next) {
        _nodebb.db.sortedSetAdd(key, 0, member.id + ':' + member.name, next);
      }, next);
    });
  })], callback);
};

Backend.setRank = function (rid, rank, callback) {
  //db.setObject(`mi:rank:${rid}`, rank, callback)
};

Backend.setRankGroup = function (rid, group, callback) {
  //db.setObjectField(`mi:rank:${rid}`, 'group', group, callback)
};

Backend.setServerRank = function (sid, rankObj, callback) {
  if (!rankObj.name) return callback(new Error('setServerRank(): Invalid Server Rank name.'));
  _async2.default.parallel([_async2.default.apply(_nodebb.db.setAdd, 'mi:server:' + sid + ':ranks', rankObj.name), _async2.default.apply(_nodebb.db.setObject, 'mi:server:' + sid + ':rank:' + rankObj.name, rankObj)], callback);
};

Backend.getServerRank = function (sid, rankName, callback) {
  _nodebb.db.getObject('mi:server:' + sid + ':rank:' + rankName, callback);
};

Backend.getServerRankPrefix = function (sid, rankName, callback) {
  _nodebb.db.getObjectField('mi:server:' + sid + ':rank:' + rankName, 'prefix', callback);
};

Backend.getServerRankRank = function (sid, rankName, callback) {
  _nodebb.db.getObjectField('mi:server:' + sid + ':rank:' + rankName, 'prefix', callback);
};

Backend.addRankGroup = function (callback) {
  _nodebb.db.getSortedSetRange('mi:groups:rank', 0, -1, callback);
};

Backend.setRankGroup = function (group, callback) {
  // db.getSortedSetRange(`group:${group}`)
  callback();
};

Backend.deleteRankGroup = function (group, callback) {
  callback();
};