'use strict';

var _nodebb = require('./nodebb');

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _backend = require('./backend');

var _backend2 = _interopRequireDefault(_backend);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Controller = module.exports = {};

// Socket event received from a user. Find the server config and socket.
function getServer(data, next) {
  if (data.sid === void 0) return next(new Error('Data with no SID was sent to the socket controller.'));

  _backend2.default.getServerConfig({ sid: data.sid }, function (err, config) {
    return next(err, { server: config, socket: config.socketid ? _nodebb.SocketIO.in(config.socketid) : null });
  });
}

Controller.sendPingToUsers = function (ping, callback) {
  _nodebb.SocketIO.in('online_users').emit('mi.ping', ping);
};

Controller.sendStatusToUsers = function (status, callback) {
  callback = callback || function () {};
  _nodebb.SocketIO.in('online_users').emit('mi.status', status);
  callback();
};

Controller.sendPlayerJoinToUsers = function (player, callback) {
  _nodebb.SocketIO.in('online_users').emit('mi.PlayerJoin', player);
};

Controller.sendPlayerQuitToUsers = function (player, callback) {
  _nodebb.SocketIO.in('online_users').emit('mi.PlayerQuit', player);
};

Controller.sendPlayerChatToUsers = function (chat, callback) {
  _nodebb.SocketIO.in('online_users').emit('mi.PlayerChat', chat);
};

Controller.sendTimeToUsers = function (timeData, callback) {
  _nodebb.SocketIO.in('online_users').emit('mi.time', timeData);
};

Controller.sendWebChatToServer = function (data, callback) {
  getServer(data, function (err, serverData) {
    if (err) return console.log(err);
    if (!serverData || !serverData.socket) return console.log(new Error('Invalid response from controller getServer()'));

    serverData.socket.emit('eventWebChat', data.chat);
  });
};

Controller.sendRewardToServer = function (rewardData, callback) {
  // TODO: Send a Reward object to the server.
};

Controller.eventGetPlayerVotes = function (socket, data, callback) {
  console.dir(data);
  getServer(data, function (err, serverData) {
    if (err) return console.log(err);
    if (!serverData || !serverData.socket) return console.log(new Error('Invalid response from controller getServer()'));

    callback();
    serverData.socket.emit('eventGetPlayerVotes', data);
  });
};

Controller.PlayerVotes = function (data, callback) {
  console.log('Got PlayerVotes');
  console.dir(data);
  // Assert parameters.
  if (!(data && data.name && data.votes)) return callback();
  callback = callback || function () {};

  var name = data.name,
      votes = data.votes,
      sid = data.sid;

  _backend2.default.getUser({ name: name }, function (err, user) {
    if (err) console.log(err);
    if (user) {
      console.log('Got user');
      console.dir(user);
      _nodebb.SocketIO.in('uid_' + user.uid).emit('mi.PlayerVotes', votes);
    }
  });

  callback();
};