'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _nodebb = require('./nodebb');

var _api = require('./api');

var _backend = require('./backend');

var _backend2 = _interopRequireDefault(_backend);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _views = require('./views');

var _views2 = _interopRequireDefault(_views);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Hooks = {
  filter: {
    scripts: {
      get: function get(data, next) {
        next(null, data);
      }
    },
    admin: {
      header: {
        build: _views2.default.buildAdminHeader
      }
    },
    config: {
      get: function get(config, next) {
        config.MinecraftIntegration = _config2.default.settings.get();
        next(null, config);
      }
    },
    widgets: {
      getWidgets: _views2.default.getWidgets
    },
    post: {
      get: function get(data, next) {
        // console.log("Post Data")
        // console.dir(data)
        next(null, data);
      },
      getPosts: function getPosts(data, next) {
        // {posts: posts, uid: uid}
        // console.log("Post Data")
        // console.dir(data)
        next(null, data);
      }
    },
    topic: {
      build: function build(data, next) {
        if (!_config2.default.settings.get('showPrefixes')) return next(null, data);

        // {req: req, res: res, templateData: data}
        if (!(data && data.templateData && data.templateData.posts && data.templateData.posts[0])) return next(null, data);

        data.templateData.prefixes = {};

        _async2.default.each(data.templateData.posts, function (post, next) {

          if (!(post && post.user && post.user.uid && data.templateData.prefixes[post.user.uid] === void 0)) return next();

          (0, _api.getUserPrefix)(post.user.uid, function (err, prefix) {
            data.templateData.prefixes[post.user.uid] = prefix;
            next();
          });
        }, function (err) {
          next(null, data);
        });
      }
    },
    user: {
      account: function account(data, callback) {
        if (!_config2.default.settings.get('showPrefixes')) return callback(null, data);
        (0, _api.getUserPrefix)(data.userData.uid, function (err, prefix) {
          data.userData.prefix = prefix;
          callback(null, data);
        });
      },
      create: function create(userData, callback) {
        console.log('Setting new user avatar for ' + userData.username);
        // var picture = pic
        // userData['picture'] = picture
        // userData['uploadedpicture'] = picture
        callback(null, userData);
      },
      profileLinks: function profileLinks(links, next) {
        links.push({
          id: 'minecraft',
          public: true,
          route: 'minecraft',
          icon: 'fa-cube',
          name: 'Minecraft'
        });
        next(null, links);
      }
    },
    users: {},
    group: {
      update: function update(data, next) {
        next(null, data);
      }
    }
  },
  action: {
    user: {
      loggedIn: function loggedIn(uid) {
        console.log('User uid ' + uid + ' logged in, checking avatar...');
        // .setUserAvatar(uid)
      },
      set: function set(data) {
        if (data.field === 'picture') {
          // .setUserAvatar(data.uid)
        }
      }
    },
    group: {
      destroy: function destroy(group) {}
    }
  }
};

function setUserAvatar(uid) {
  _nodebb.User.getUserFields(uid, ['uuid', 'username', 'picture'], function (err, fields) {
    if (err) {
      console.log('Tried to check the avatar of user uid ' + uid + ', but got an error:', err);
    } else if (!fields.uuid) {
      console.log('User uid ' + uid + ' does not have a UUID assigned, adding to refresh queue.');
      // TODO: Add a refresh queue. :P
    } else {
      var picture = _config2.default.settings.get('avatarCDN').replace('{username}', fields.username);
      if (picture !== fields.picture) {
        console.log('Changing avatar for user ' + uid);
        _nodebb.User.setUserFields(uid, { picture: picture, uploadedpicture: picture }, function (err) {
          if (err) {
            console.log(err);
          }
        });
      } else {
        console.log('User uid ' + uid + ' has the correct avatar!');
      }
    }
  });
}

exports.default = Hooks;