'use strict';

var _api = require('./api');

var _nodebb = require('./nodebb');

var _nodebb2 = _interopRequireDefault(_nodebb);

var _backend = require('./backend');

var _backend2 = _interopRequireDefault(_backend);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Registration = module.exports = {};


Registration.register = function (data, next) {

  // Assert parameters.
  if (!(data && data.id && data.name && data.pkey)) return next(new Error('FAILDATA'));

  // Local vars.
  var sid = data.sid,
      id = data.id,
      name = data.name,
      key = data.pkey;

  console.log('Got "/register" command from Minecraft server ' + sid + '.\nPlayer ' + data.name + ' is attempting to register with player key ' + key);

  (0, _api.getPlayerKeyUID)({ key: key, name: name }, function (err, uid) {
    if (err || !uid) {
      return next(new Error('FAILKEY'));
    }

    _async2.default.parallel([
    // TODO: Create an account renaming option.

    // Link the accounts.
    _async2.default.apply(_backend2.default.linkUuidtoUid, id, uid),

    // Change the register key.
    _async2.default.apply(_api.resetPlayerKey, { uid: uid })], function (err) {
      if (err) {
        console.log('Register err for UID ' + uid + ': ' + err);
        next(new Error('FAILDB'));
      } else {
        console.log('Set the Minecraft UUID for UID ' + uid + ' to ' + id);
        next(null, { result: 'REGISTER' });
      }
    });
  });
};

Registration.resetPlayerKey = function (data, next) {
  if (!(data && data.uid && data.sender)) return next(new Error('Bad data sent to Registration.resetPlayerKey()'));
  if (!(data.uid === data.sender)) return next(new Error("Can't reset others' player keys."));

  (0, _api.resetPlayerKey)({ uid: data.uid }, next);
};