'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (app, middleware, router) {
  // TODO: Add slugs
  // db.sortedSetScore('mi:servers:slugs')

  router.get('/mc/chat', middleware.buildHeader, function (req, res) {
    res.render('mc/chat', { sid: 0 });
  });

  router.get('/mc/ranks', middleware.buildHeader, Controllers.renderRanks);
  router.get('/api/mc/ranks', Controllers.renderRanks);

  function render(req, res, next) {
    res.render('admin/plugins/minecraft-integration', {});
  }

  router.get('/admin/plugins/minecraft-integration', middleware.admin.buildHeader, render);
  router.get('/api/admin/plugins/minecraft-integration', render);
  router.get('/minecraft-integration/config', function (req, res) {
    res.status(200);
  });

  // Initialize socket namespace
  _nodebb.SocketPlugins.MinecraftIntegration = {};

  // Define API Functions

  function callHTTP(method, respondWith, req, res, next) {
    var data = {};

    for (var param in req.params) {
      data[param] = req.params[param];
    }

    method(data, function (err, response) {
      if (err) {
        console.log(err);
        return res.sendStatus(404);
      }
      switch (respondWith) {
        default:
        case 'JSON':
          res.json(response);
          break;
        case 'Buffer':
          if (req.get('If-Modified-Since') === response.modified) {
            res.sendStatus(304);
          } else {
            res.writeHead(200, {
              'Cache-Control': 'private',
              'Last-Modified': response.modified,
              'Content-Type': 'image/png'
            });
            res.end(response.buffer || new Buffer(response.base, 'base64'), 'binary');
          }
          break;
        case 'String':
          res.writeHead(200, {
            'Content-Type': 'text/plain'
          });
          res.end(response.base || response, 'binary');
          break;
      }
    });
  }

  function addToAPI(method, name, path) {
    var respondWith = arguments.length <= 3 || arguments[3] === undefined ? 'JSON' : arguments[3];

    router.get('/api/minecraft-integration/' + path, function (req, res, next) {
      callHTTP(method, respondWith, req, res, next);
    });

    _nodebb.SocketPlugins.MinecraftIntegration[name] = function (socket, data, next) {
      data.sender = socket.uid;
      method(data, next);
    };
  }

  function addToWriteAPI(method, name, path) {
    var respondWith = arguments.length <= 3 || arguments[3] === undefined ? 'JSON' : arguments[3];

    // Write using HTTP.
    if (!!path) {

      // TODO: Should be a PUT request.
      router.get('/api/minecraft-integration/write/:key/' + path, function (req, res, next) {

        if (!(req.params && req.params.key)) return res.json(new Error('No API key.'));

        _backend2.default.getSidUsingAPIKey(req.params.key, function (err, sid) {

          if (err) return res.json({ error: 'Invalid API key.' });
          if (!sid) return res.json({ error: 'Invalid API key.' });

          req.params.sid = sid;

          callHTTP(method, respondWith, req, res, next);
        });
      });
    }

    // Write using sockets.
    _nodebb.SocketPlugins.MinecraftIntegration[name] = function (socket, data, next) {
      if (!(data && data.key)) return next(new Error('No API key.'));

      // TODO: Proper logger.
      // Log.info('Write API connection attempt with key ' + data.key)

      // Verify API key.
      _backend2.default.getSidUsingAPIKey(data.key, function (err, sid) {
        if (err || !sid) {
          // TODO
          console.log('Invalid API key for ' + data.sid);
          return next('Invalid API key.', {});
        }

        data.sid = sid;

        // If event has a player id, trim it to Mojang's format.
        if (data.id) data.id = _utils2.default.trimUUID(data.id);

        // Set the socket.id so that we can send events back to the server.
        _nodebb.db.setObjectField('mi:server:' + data.sid + ':config', 'socketid', socket.id);

        // Detach key from data response.
        delete data.key;

        method(data, next);
      });
    };
  }

  // Read API

  addToAPI(_backend2.default.getServers, 'getServers', 'servers');

  addToAPI(API.getServerStatus, 'getServerStatus', 'server/:sid');
  addToAPI(_backend2.default.getRecentPings, 'getRecentPings', 'server/:sid/pings');
  addToAPI(_backend2.default.getRecentPings, 'getRecentPings', 'server/:sid/pings/:last');
  addToAPI(_backend2.default.getServerIcon, 'getServerIcon', 'server/:sid/icon', 'Buffer');
  addToAPI(_backend2.default.getServerPlugins, 'getServerPlugins', 'server/:sid/plugins');

  addToAPI(_chat2.default.getChat, 'getChat', 'server/:sid/chat');
  addToAPI(_chat2.default.getChat, 'getChat', 'server/:sid/chat/:chats');
  addToAPI(_chat2.default.getChat, 'getChat', 'server/:sid/chat/:chats/:max');

  addToAPI(_backend2.default.getAvatars, 'getAvatars', 'avatars');
  addToAPI(_backend2.default.getAvatar, 'getAvatar', 'avatar/:name', 'String');
  addToAPI(_backend2.default.getAvatar, 'getAvatar', 'avatar/:name/:size', 'Buffer');

  // Tracked player objects.
  addToAPI(API.getPlayers, 'getPlayers', 'players');
  addToAPI(API.getPlayer, 'getPlayer', 'players/name/:name');
  addToAPI(API.getPlayer, 'getPlayer', 'players/uuid/:id');
  addToAPI(API.getPlayer, 'getUserPlayers', 'players/uid/:uid');

  addToAPI(API.getUsers, 'getUsers', 'users');
  addToAPI(API.getUser, 'getUser', 'users/uuid/:id');
  addToAPI(API.getUser, 'getUser', 'users/name/:name');

  addToAPI(_registration2.default.resetPlayerKey, 'resetPlayerKey', 'users/reset/:uid');

  addToAPI(_config2.default.getSettings, 'getSettings', 'settings');
  addToAPI(_config2.default.getSettings, 'getSettings', 'settings/:key');

  addToAPI(API.getPlayerPrefix, 'getPlayerPrefix', 'players/name/:name/prefix');
  addToAPI(API.getUserPrefix, 'getUserPrefix', 'users/uid/:uid/prefix');

  addToAPI(_config2.default.getAvatarUrl, 'getAvatarUrl', 'avatar');

  addToAPI(_utils2.default.getKey, 'getKey', 'key');

  addToAPI(_backend2.default.getPlaytimes, 'getPlaytimes', 'playtimes');
  addToAPI(_backend2.default.getTopPlayersByPlaytimes, 'getTopPlayersByPlaytimes', 'playtimes/top');
  addToAPI(_backend2.default.getTopPlayersByPlaytimes, 'getTopPlayersByPlaytimes', 'playtimes/top/:show');

  // Write API

  addToWriteAPI(API.updateServerStatus, 'eventStatus');

  addToWriteAPI(API.eventPlayerChat, 'eventPlayerChat', 'chat/:id/:name/:message');
  addToWriteAPI(API.eventPlayerJoin, 'eventPlayerJoin', 'join/:id/:name');
  addToWriteAPI(API.eventPlayerQuit, 'eventPlayerQuit', 'quit/:id/:name');
  addToWriteAPI(API.writeOfflinePlayers, 'writeOfflinePlayers', 'offlineplayers');
  addToWriteAPI(API.writeRanks, 'writeRanks', 'ranks');
  addToWriteAPI(API.writeRanksWithMembers, 'writeRanksWithMembers', 'ranks-with-members');

  addToWriteAPI(_registration2.default.register, 'commandRegister');

  addToWriteAPI(_controller2.default.PlayerVotes, 'PlayerVotes');

  // Request API
  _nodebb.SocketPlugins.MinecraftIntegration.eventWebChat = API.eventWebChat;
  _nodebb.SocketPlugins.MinecraftIntegration.eventGetPlayerVotes = _controller2.default.eventGetPlayerVotes;
};

var _nodebb = require('../nodebb');

var _api = require('../api');

var API = _interopRequireWildcard(_api);

var _backend = require('../backend');

var _backend2 = _interopRequireDefault(_backend);

var _config = require('../config');

var _config2 = _interopRequireDefault(_config);

var _controller = require('../controller');

var _controller2 = _interopRequireDefault(_controller);

var _controllers = require('../controllers');

var Controllers = _interopRequireWildcard(_controllers);

var _utils = require('../utils');

var _utils2 = _interopRequireDefault(_utils);

var _chat = require('../chat');

var _chat2 = _interopRequireDefault(_chat);

var _registration = require('../registration');

var _registration2 = _interopRequireDefault(_registration);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }