'use strict';

var _nodebb = require('./nodebb');

var _api = require('./api');

var _backend = require('./backend');

var _backend2 = _interopRequireDefault(_backend);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _controller = require('./controller');

var _controller2 = _interopRequireDefault(_controller);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

var _nconf = require('nconf');

var _nconf2 = _interopRequireDefault(_nconf);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var updateTime = 0;
var scheduler = void 0;

var Updater = module.exports = {};

var uuidsNeedingUpdates = [];

Updater.updateUuids = function (uuids) {
  if (!Array.isArray(uuids)) return;

  for (var i in uuids) {
    if (uuidsNeedingUpdates.indexOf(uuids[i]) === -1) {
      uuidsNeedingUpdates.push(uuids[i]);
    }
  }
};

function updatePlayers() {
  _async2.default.each(uuidsNeedingUpdates, function (id, next) {

    var key = 'yuuid:' + id;

    // Get the name from Mojang.
    _utils2.default.getName(id, function (err, name) {

      // Return if db error.
      if (err) return next(err);

      // Update the player object.
      _nodebb.db.setObjectField(key, 'name', name);
      _nodebb.db.setObjectField(key, 'lastupdate', Date.now());

      // Update player name cache.
      _nodebb.db.set('mi:name:' + name, id);
      _nodebb.db.expire('mi:name:' + name, _config2.default.getPlayerExpiry());

      // Sort by last update.
      _nodebb.db.isSortedSetMember('yuuid:sorted', id, function (err, isMember) {
        if (!err && !isMember) _nodebb.db.sortedSetAdd('yuuid:sorted', Date.now(), id);
      });

      // Add to playtime cache.
      _nodebb.db.isSortedSetMember('yuuid:playtime', id, function (err, isMember) {
        if (!err && !isMember) _nodebb.db.sortedSetAdd('yuuid:playtime', 0, id);
      });

      next();
    });
  }, function () {
    uuidsNeedingUpdates = [];
  });
}

_nodebb.pubsub.on('meta:reload', function () {
  if (scheduler) clearTimeout(scheduler);
});

Updater.init = function () {
  if (scheduler) clearTimeout(scheduler);
  // Only start on primary node.
  if (!(_nconf2.default.get('isPrimary') === 'true' && !_nconf2.default.get('jobsDisabled'))) return;
  scheduler = setTimeout(Updater.updateServers, 60000);
};

Updater.updateServers = function () {

  // Get the current minute.
  updateTime = Math.round(Date.now() / 60000) * 60000;

  // Remove old avatars from cache.
  _backend2.default.clearOldAvatars();

  _backend2.default.getServersConfig({}, function (err, configs) {

    configs.forEach(function (config) {

      (0, _api.getServerStatus)(config, function (err, status) {

        if (err) {
          _winston2.default.info('Error getting status for server ' + config.name);
          return resetStatus(status);
        }

        if (!status) return _winston2.default.info('No status recorded for server ' + config.name);

        if (!status.updateTime || parseInt(status.updateTime, 10) + 1000 * 60 < updateTime) {
          _winston2.default.info('The Minecraft server ' + config.name + ' is not connected to forum.');
          _winston2.default.info('Use the server\'s API key in the command "/nodebb key {key}"');
          return resetStatus(status);
        }
      });
    });
  });

  // Update players.
  updatePlayers();

  // Schedule next update.
  Updater.init();
};

function resetStatus() {
  var status = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  status.isServerOnline = '0';
  status.updateTime = updateTime;
  status.players = '[]';
  status.onlinePlayers = '0';
  status.tps = '0';

  _backend2.default.updateServerStatus(status);
  _controller2.default.sendStatusToUsers(status);
}