'use strict';

var _nodebb = require('./nodebb');

var _api = require('./api');

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _backend = require('./backend');

var _backend2 = _interopRequireDefault(_backend);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var nconf = require.main.require('nconf');

var Views = module.exports = {};

var app = void 0;

Views.init = function (_app, middleware, router) {

  app = _app;

  function renderMinecraftProfile(req, res) {
    var payload = {};

    _async2.default.waterfall([_async2.default.apply(_nodebb.User.getUidByUserslug, req.params.user), _async2.default.apply(_nodebb.User.getUserData), function (userData, next) {

      // TODO: Attach profile info.
      // if (!(userData && userData.yuuid)) return next()
      // NodeBB.db.getObject('yuuid:' + userData.yuuid, next)

      payload = userData;

      if (req.uid !== parseInt(userData.uid, 10)) {
        payload.isSelf = false;
        return next(null, userData.uid);
      } else {
        payload.isSelf = true;
      }

      (0, _api.getPlayerKey)({ uid: req.uid }, function (err, result) {
        if (err) return next(err);
        payload.playerKey = result.key;
        return next(err, userData.uid);
      });
    }, function (uid, next) {
      (0, _api.getUserLinkedPlayers)(uid, function (err, players) {
        if (err || !players || !players.length) {
          payload.hasPlayers = false;
        } else {
          payload.hasPlayers = true;
          payload.players = players;
        }
        next();
      });
    }], function (err) {
      if (err) console.log(err);

      payload.title = req.params.user;

      res.render('account/minecraft', payload);
    });
  }

  router.get('/api/user/:user/minecraft', renderMinecraftProfile);
  router.get('/user/:user/minecraft', middleware.buildHeader, renderMinecraftProfile);

  router.get('/minecraft/register', redirectRegister);
  router.get('/mc/register', redirectRegister);
};

function redirectRegister(req, res) {
  if (!req.uid) {
    res.redirect('/register');
  } else {
    _nodebb.User.getUserField(req.uid, 'userslug', function (err, slug) {
      res.redirect('/user/' + slug + '/minecraft');
    });
  }
}

Views.buildAdminHeader = function (custom_header, next) {
  custom_header.plugins.push({
    'route': '/plugins/minecraft-integration',
    'icon': 'fa-cube',
    'name': 'Minecraft Integration'
  });

  return next(null, custom_header);
};

Views.getWidgets = function (widgets, next) {
  var _widgets = [{ widget: 'mi-chat', name: 'Minecraft Chat', content: 'admin/widgets/chat.tpl', description: 'Shows a shoutbox-like area connected to the in-game chat.' }, { widget: 'mi-map', name: 'Minecraft Mini Map', content: 'admin/widgets/map.tpl', description: 'Shows a small Map.' }, { widget: 'mi-players-graph', name: 'Minecraft Players Graph', content: 'admin/widgets/players-graph.tpl', description: 'Shows a graph showing players over time.' }, { widget: 'mi-players-grid', name: 'Minecraft Players Grid', content: 'admin/widgets/players-grid.tpl', description: 'Shows the avatars of all online players or a group of specific players.' }, { widget: 'mi-status', name: 'Minecraft Server Status', content: 'admin/widgets/status.tpl', description: 'Lists information on a Minecraft server.' }, { widget: 'mi-top-graph', name: 'Minecraft Top Players Graph', content: 'admin/widgets/top-graph.tpl', description: "A graphic chart (Pie, Donut, or Bar) representing the top players' based on a specific statistic." }, { widget: 'mi-top-list', name: 'Minecraft Top Players List', content: 'admin/widgets/top-list.tpl', description: "Lists avatars representing the top players' based on a specific statistic." }, { widget: 'mi-tps-graph', name: 'Minecraft TPS Graph', content: 'admin/widgets/tps-graph.tpl', description: 'Shows the approximate tick time of the server over time.' }, { widget: 'mi-vote-list', name: 'Minecraft Vote List', content: 'admin/widgets/vote-list.tpl', description: 'Links to sites where players can vote for the server.' }
  // { widget: "mi-directory",     name: "Minecraft Player Directory",  content: 'admin/widgets/directory.tpl',     description: "Find and view information on players." }
  // { widget: "mi-gallery",       name: "Minecraft Gallery",           content: 'admin/widgets/gallery.tpl',       description: "A gallery of player uploaded screenshots." }
  // { widget: "mi-ping-graph",    name: "Minecraft Ping Graph",        content: 'admin/widgets/players-graph.tpl', description: "Shows the ping (network latency) of the server over time." }
  ];

  _backend2.default.getServersConfig({}, function (err, servers) {

    if (err) console.log('TODO: add an error for widget load failure.');

    _async2.default.each(_widgets, function (widget, next) {
      app.render(widget.content, { servers: servers }, function (err, content) {
        _nodebb.translator.translate(content, function (content) {
          widget.content = content;
          next();
        });
      });
    }, function (err) {
      widgets = widgets.concat(_widgets);
      next(null, widgets);
    });
  });
};

Views.modifyTemplates = function (callback) {
  callback = callback || function () {};

  var tplPath = _path2.default.join(nconf.get('base_dir'), 'public/templates/account/profile.tpl');

  _async2.default.parallel({
    original: function original(next) {
      _fs2.default.readFile(tplPath, next);
    }
  }, function (err, tpls) {
    if (err) {
      return callback(err);
    }

    var tpl = tpls.original.toString();

    if (!tpl.match('{prefix}')) {
      // Persona
      tpl = tpl.replace('<h1 class="fullname"><!-- IF fullname -->{fullname}<!-- ELSE -->{username}<!-- ENDIF fullname --></h1>', '<span class="h1 prefix" <!-- IF !prefix -->style="display:none;"<!-- ENDIF !prefix -->>{prefix} {username}</span><h1 class="fullname"><!-- IF fullname -->{fullname}<!-- ELSE -->{username}<!-- ENDIF fullname --></h1>');

      // Vanilla/Lavender
      tpl = tpl.replace('			<i component="user/status" class="fa fa-circle status {status}" title="[[global:{status}]]"></i>', '<span class="h4" <!-- IF !prefix -->style="display:none;"<!-- ENDIF !prefix -->>{prefix}</span><br><i component="user/status" class="fa fa-circle status {status}" title="[[global:{status}]]"></i>');
    }

    _fs2.default.writeFile(tplPath, tpl, callback);
  });
};