'use strict';

var _nodebb = require('./nodebb');

var _api = require('./api');

var _backend = require('./backend');

var _backend2 = _interopRequireDefault(_backend);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _chat = require('./widgets/chat');

var _directory = require('./widgets/directory');

var _gallery = require('./widgets/gallery');

var _map = require('./widgets/map');

var _pingGraph = require('./widgets/ping-graph');

var _playersGraph = require('./widgets/players-graph');

var _playersGrid = require('./widgets/players-grid');

var _status = require('./widgets/status');

var _topGraph = require('./widgets/top-graph');

var _topList = require('./widgets/top-list');

var _tpsGraph = require('./widgets/tps-graph');

var _voteList = require('./widgets/vote-list');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Widgets = module.exports = {};

var app = void 0;

Widgets.init = function (_app) {
  app = _app;
};

Widgets.renderChat = function (widget, callback) {
  render(_chat.render, 'chat', widget, callback);
};
Widgets.renderDirectort = function (widget, callback) {
  render(_directory.render, 'directory', widget, callback);
};
Widgets.renderGallery = function (widget, callback) {
  render(_gallery.render, 'gallery', widget, callback);
};
Widgets.renderMap = function (widget, callback) {
  render(_map.render, 'map', widget, callback);
};
Widgets.renderPingGraph = function (widget, callback) {
  render(_pingGraph.render, 'ping-graph', widget, callback);
};
Widgets.renderPlayersGraph = function (widget, callback) {
  render(_playersGraph.render, 'players-graph', widget, callback);
};
Widgets.renderPlayersGrid = function (widget, callback) {
  render(_playersGrid.render, 'players-grid', widget, callback);
};
Widgets.renderStatus = function (widget, callback) {
  render(_status.render, 'status', widget, callback);
};
Widgets.renderTopGraph = function (widget, callback) {
  render(_topGraph.render, 'top-graph', widget, callback);
};
Widgets.renderTopList = function (widget, callback) {
  render(_topList.render, 'top-list', widget, callback);
};
Widgets.renderTPSGraph = function (widget, callback) {
  render(_tpsGraph.render, 'tps-graph', widget, callback);
};
Widgets.renderVoteList = function (widget, callback) {
  render(_voteList.render, 'vote-list', widget, callback);
};

function formatWidget(widget, callback) {

  // API checks for invalid SIDs.
  (0, _api.getServerStatus)({ sid: widget.data.sid }, function (err, status) {
    if (err) return callback(err);

    for (var p in status) {
      widget.data[p] = status[p];
    }if (widget.data.parseFormatCodes == 'on' ? true : false) {
      widget.data.name = _utils2.default.parseMCFormatCodes(status.name);
      widget.data.motd = _utils2.default.parseMCFormatCodes(status.motd);
    } else {
      // TODO: Remove formatting codes.
    }

    widget.data.title = widget.data.title.replace(/\{\{motd\}\}/, widget.data.motd);
    widget.data.title = widget.data.title.replace(/\{\{name\}\}/, widget.data.name);
    widget.data.container = widget.data.container.replace(/\{\{motd\}\}/, widget.data.motd);
    widget.data.container = widget.data.container.replace(/\{\{name\}\}/, widget.data.name);

    widget.data.colorTitle = widget.data.colorTitle || 'unset';
    widget.data.colorLabels = widget.data.colorLabels || 'unset';
    widget.data.colorText = widget.data.colorText || 'unset';

    callback(null, widget.data);
  });
}

function render(renderWidget, type, widget, callback) {
  _async2.default.waterfall([_async2.default.apply(formatWidget, widget), _async2.default.apply(renderWidget)], function (err, data) {

    if (err) return callback();

    if (data.container) data.container = data.container.replace('class="panel-body"', 'class="panel-body" style="padding:0;"');

    app.render('widgets/' + type, data, function (err, html) {
      _nodebb.translator.translate(html, function (translatedHTML) {
        callback(null, translatedHTML);
      });
    });
  });
}